<?

use Bitrix\Main\Application;
use Bitrix\Main\Config\Option;
use Bitrix\Main\Localization\Loc;
use Bitrix\Sale\Basket;
use Bitrix\Sale\Order;
use Bitrix\Main\Web\HttpClient;
use Bitrix\Main\Web\Json;
use Bitrix\Main\Data\Cache;
use Bitrix\Main\Text\Encoding;
use Bitrix\Main\IO\Directory;
use Bitrix\Main\Page\Asset;
use Bitrix\Main\ArgumentException;

class Log{
	static $LOG_DIRECTORY = null;
	static $LOG_FILE = null;
	function __construct(){
		self::$LOG_FILE = 'boxberrydelivery.log';
		self::$LOG_DIRECTORY = $_SERVER['DOCUMENT_ROOT']. '/bitrix/cache/log/';
		if (Option::get(CBoxberry::$moduleId, 'BB_LOG') == 'Y'){
			if(!file_exists(self::$LOG_DIRECTORY)){
				mkdir(self::$LOG_DIRECTORY,0777,1);
			}
		}else{
			if (file_exists(self::$LOG_DIRECTORY.self::$LOG_FILE)){
				unlink(self::$LOG_DIRECTORY.self::$LOG_FILE);
			}
		}
	}
	function save($data){
            file_put_contents(self::$LOG_DIRECTORY . self::$LOG_FILE, print_r($data, 1), FILE_APPEND);
	}
}

class CBoxberry
{
    public static $moduleId = "up.boxberrydelivery";
    public static $apiToken;
    public static $apiUrl;
    public static $log = false;

    public static function initApi()
    {
        $apiToken = trim(Option::get(self::$moduleId, 'API_TOKEN'));
        if (!empty($apiToken)) {
            self::$apiToken = $apiToken;
        } else {
            return false;
        }

        self::$log = new Log();

        self::$apiUrl = trim(Option::get(self::$moduleId, 'API_URL'));

        return true;
    }

    public static function httpRequest($url)
    {
        $httpClient = new HttpClient();
        $httpClient->get($url);
        if ($httpClient->getStatus() !== 200) {
            return false;
        }

        return $httpClient->getResult();
    }

    public static function apiGet($url, $cacheTime = 0)
    {
        if (!$cacheTime) {
            return self::httpRequest($url);
        }

        $cache = Cache::createInstance();
        $key = md5($url);

        if ($cache->initCache($cacheTime, $key)) {
            $vars = $cache->getVars();
            return $vars[$key];
        }

        if ($cache->startDataCache()) {
            $value = self::httpRequest($url);
            $cache->endDataCache([$key => $value]);
            return $value;
        }

        return false;
    }

    public static function updateDataAgent()
    {
        if (!self::initApi()) {
            return false;
        }

        $cacheTime = 10800;
        $methods = [
            'ListCities',
            'ListZips',
            'ListPointsShort',
            'WidgetSettings',
            'GetKeyIntegration'
        ];

        foreach ($methods as $method) {
            self::methodExec($method, $cacheTime);
        }

        return 'CBoxberry::updateDataAgent();';
    }

    public static function methodExec($method, $cacheTime = 0, $params = null)
    {
        $urlToken = '?token=' . self::$apiToken;

        if (isset ($method) && !empty($method)) {
            if (is_array($params)) {
                $params = implode('&', $params);
                $urlParams = '&' . $params;
            } else {
                $urlParams = empty($params) ? '' : '&' . $params;
            }

            $execString = self::$apiUrl . $urlToken . '&method=' . $method . $urlParams;

            try {
                $data = Json::decode(self::apiGet($execString, $cacheTime));
            } catch (ArgumentException $e) {
                return false;
            }

            if (Option::get(self::$moduleId, 'BB_LOG') === 'Y') {
                self::$log->save($method . $urlParams);
                self::$log->save($data);
            }

            return $data;
        }

        return false;
    }

    public static function methodExecPost($method, $inData = null)
    {
        $postMethod = new HttpClient();
        $postMethod->post(
            self::$apiUrl,
            [
                'token' => self::$apiToken,
                'method' => $method,
                'sdata' => Json::encode($inData, JSON_UNESCAPED_UNICODE)
            ]
        );

        try {
            $outData = Json::decode($postMethod->getResult());
        } catch (ArgumentException $e) {
            return false;
        }

        if (Option::get(self::$moduleId, 'BB_LOG') === 'Y') {
            self::$log->save($inData);
            self::$log->save($outData);
        }

        return $outData;
    }

    public static function saveFilesFromApi($request, $orderId)
    {
        $pdfDir = '/bitrix/pdf/';
        $serverPdfDir = Application::getDocumentRoot() . $pdfDir;
        $pathToPdf = $serverPdfDir . $orderId . '-' . date('d_m_Y-h_i_s') . '.pdf';
        $linkToPdf = $pdfDir . $orderId . '-' . date('d_m_Y-h_i_s') . '.pdf';
        if (!Directory::isDirectoryExists($serverPdfDir)) {
            Directory::createDirectory($serverPdfDir);
        }

        (new HttpClient())->download($request, $pathToPdf);

        return $linkToPdf;
    }

    public static function GetFullOrderData($ORDER_ID)
    {
        if(intval($ORDER_ID) <= 0) return false;

        $order = CSaleOrder::GetByID($ORDER_ID);
        $bxbOrderInfo = CBoxberryOrder::GetByOrderId($ORDER_ID);
        $order["PVZ_CODE"] = $bxbOrderInfo["PVZ_CODE"];

        if(!$order) return false;

        $dbProps = CSaleOrderPropsValue::GetOrderProps($ORDER_ID);
        while($prop = $dbProps->Fetch())
        {
            $order['PROPS'][$prop['ORDER_PROPS_ID']] = $prop;
        }
        $order['ITEMS'] = array();

        $dbBasket = CSaleBasket::GetList(Array('ID'=>'ASC'), Array('ORDER_ID'=>$ORDER_ID));
        $BB_WEIGHT = COption::GetOptionString(self::$moduleId, 'BB_WEIGHT');
        while ($arItem = $dbBasket->Fetch())
        {
            $order['ITEMS'][] = $arItem;
            $order['PACKAGE_WEIGHT']+= $arItem["QUANTITY"] * ($arItem['WEIGHT'] == '0.00' ?  $BB_WEIGHT : $arItem['WEIGHT']);
        }

        $order['WIDTH'] = $bxbOrderInfo['BB_WIDTH'];
        $order['HEIGHT'] = $bxbOrderInfo['BB_HEIGHT'];
        $order['DEPTH'] = $bxbOrderInfo['BB_DEPTH'];

        return $order;
    }

    public static function MakePropsArray($order)
    {
    	$arReturn = array();

			$arReturn['VID'] = (($order['DELIVERY_ID'] == 'boxberry:PVZ' || $order['DELIVERY_ID'] == 'boxberry:PVZ_COD') ? 1 : 2);
			$arOptFields = array(
				'BB_FIO'=> Option::get(self::$moduleId, 'BB_FIO'),
				'BB_CONTACT_PERSON'=> Option::get(self::$moduleId, 'BB_CONTACT_PERSON'),
				'BB_PHONE'=> Option::get(self::$moduleId, 'BB_PHONE'),
				'BB_EMAIL'=> Option::get(self::$moduleId, 'BB_EMAIL'),
				'BB_ZIP'=> Option::get(self::$moduleId, 'BB_ZIP'),
				'BB_LOCATION'=> Option::get(self::$moduleId, 'BB_LOCATION'),
				'BB_ADDRESS'=> Option::get(self::$moduleId, 'BB_ADDRESS'),
				'BB_JUR_ADDRESS'=> Option::get(self::$moduleId, 'BB_JUR_ADDRESS'),
				'BB_COMPANY_NAME'=> Option::get(self::$moduleId, 'BB_COMPANY_NAME'),
				'BB_INN'=> Option::get(self::$moduleId, 'BB_INN'),
				'BB_KPP'=> Option::get(self::$moduleId, 'BB_KPP'),
			);
			foreach($arOptFields as $key=> $optName)
			{
				foreach((array)$order['PROPS'] as $cur_prop)
				{
					if ($optName==$cur_prop['CODE'])
					{
						$arReturn[$key] = $cur_prop["VALUE"];
					}
				}
			}
		return (count($arReturn) > 0 ? $arReturn : FALSE);
	}

    public static function changeAddress ($ORDER_ID=NULL, $address=NULL){
		if (!empty($address)){
			$dbProps = CSaleOrderPropsValue::GetOrderProps($ORDER_ID);
			$address_prop_bb = Option::get(self::$moduleId, 'BB_ADDRESS');
			$location_prop_bb = Option::get(self::$moduleId, 'BB_LOCATION');
			if (strtoupper(LANG_CHARSET) != 'UTF-8'){
				$address = mb_convert_encoding ($address,'CP1251','UTF-8');
			}

			while($prop = $dbProps->Fetch())
			{
				if ($prop['CODE'] == $address_prop_bb)
				{
					CSaleOrderPropsValue::Update($prop['ID'], array("ORDER_ID"=>$ORDER_ID, "CODE"=>$prop['CODE'] ,"VALUE"=>$address));
				}
			}
		}

	}

    public static function updatePvz($pvzId, $orderId = null, $address = null)
    {
        $arFields = array(
            'ORDER_ID' => $orderId,
            'PVZ_CODE' => Encoding::convertEncoding($pvzId, 'UTF-8', LANG_CHARSET),

        );
        CBoxberryOrder::Update($orderId, $arFields);
        self::changeAddress($orderId, $address);
    }

    public static function saveadminPvz($pvzId, $orderId = null, $address = null)
    {
        $currentDate = date('d.m.Y H:i:s');
        $arFields = array(
            'PVZ_CODE' => Encoding::convertEncoding($pvzId, 'UTF-8', LANG_CHARSET),
            'STATUS_DATE' => $currentDate,
        );
        CBoxberryOrder::Update($orderId, $arFields);
        self::changeAddress($orderId, $address);
    }

    public static function savePvz($pvzId)
    {
        $_SESSION['selPVZ'] = Encoding::convertEncoding($pvzId, 'UTF-8', LANG_CHARSET);
    }

    public static function removePvz()
    {
        unset($_SESSION['selPVZ']);
    }

    public static function checkPvz()
    {
        $_SESSION['checkPVZ'] = true;
    }

    public static function disableCheckPvz()
    {
        unset($_SESSION['checkPVZ']);
    }

    public static function addWidgetJs()
    {
        $widgetUrl = trim(Option::get(self::$moduleId, 'WIDGET_URL'));

        if (!$widgetUrl) {
            $widgetUrl = 'https://points.boxberry.de/js/boxberry.js';
        }

        Asset::getInstance()->addJs($widgetUrl);
    }

}

?>