<?

use Bitrix\Main\Config\Option;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Loader;
use Bitrix\Sale\Order;

Loader::includeModule('sale');

class CBoxberryParsel
{
    public static $moduleId = 'up.boxberrydelivery';

    public function parselCreate($orderId)
    {
        if (!CBoxberry::initApi()) {
            return array('ERROR' => Loc::getMessage('WRONG_API_CONNECT'));
        }

        if ((int)$orderId <= 0) {
            return array('ERROR' => 'Invalid argument.');
        }

        if (!$order = CBoxberry::GetFullOrderData($orderId)) {
            return array('ERROR' => 'Order not found.');
        }

        if (!$arProps = CBoxberry::MakePropsArray($order)) {
            return array('ERROR' => 'Wrong module settings.');
        }

        $orderNumber = (Option::get(self::$moduleId, 'BB_ACCOUNT_NUMBER') === 'Y' ? $order['ACCOUNT_NUMBER'] : $order['ID']);
        $location = $arProps['BB_LOCATION'];
        $arLocation = CSaleLocation::GetByID($location, 'ru');
        $cityName = $arLocation['CITY_NAME'];
        $regionName = $arLocation['REGION_NAME'];

        $sdata = array(
            'order_id' => $orderNumber,
            'price' => $order['PRICE'],
            'payment_sum' => ($order['PAYED'] === 'Y' ? 0 : $order['PRICE']),
            'delivery_sum' => $order['PRICE_DELIVERY'],
            'vid' => $arProps['VID'],
        );

        $sdata['shop'] = array(
            'name' => ($order['PVZ_CODE'] ?: ''),
            'name1' => ''
        );
        $bxbOptions['bb_paid_person_jur'] = Option::get(self::$moduleId, 'BB_PAID_PERSON_JUR');
        $bxbOptions['bb_paid_person_jur'] = (!empty($bxbOptions['bb_paid_person_jur']) ? $bxbOptions['bb_paid_person_jur'] : 2);

        if ($order['PERSON_TYPE_ID'] === $bxbOptions['bb_paid_person_jur']) {
            $sdata['customer'] = array(
                'fio' => $arProps['BB_CONTACT_PERSON'],
                'phone' => $arProps['BB_PHONE'],
                'email' => $arProps['BB_EMAIL'],
                'name' => $arProps['BB_COMPANY_NAME'],
                'address' => $arProps['BB_JUR_ADDRESS'],
                'inn' => $arProps['BB_INN'],
                'kpp' => $arProps['BB_KPP']
            );
        } else {
            $sdata['customer'] = array(
                'fio' => $arProps['BB_FIO'],
                'phone' => $arProps['BB_PHONE'],
                'email' => $arProps['BB_EMAIL']
            );
        }

        $sdata['weights'] = array(
            'weight' => ($order['PACKAGE_WEIGHT'] < 5 ? 5 : ceil($order['PACKAGE_WEIGHT'])),
            'x' => ((is_null($order['HEIGHT']) || $order['HEIGHT'] < 1) ? '' : $order['HEIGHT']),
            'y' => ((is_null($order['WIDTH']) || $order['WIDTH'] < 1) ? '' : $order['WIDTH']),
            'z' => ((is_null($order['DEPTH']) || $order['DEPTH'] < 1) ? '' : $order['DEPTH'])
        );

        if ($arProps['VID'] === 2) {
            $sdata['kurdost'] = array(
                'index' => '',
                'citi' => $cityName . ' ' . $regionName,
                'addressp' => $arProps['BB_ADDRESS'],
                'timep' => '09:00 - 20:00'
            );
        }

        $sdata['items'] = array();
        foreach ($order['ITEMS'] as $item) {
            $sdata['items'][] =
                array(
                    'id' => $item['PRODUCT_ID'],
                    'name' => $item['NAME'],
                    'UnitName' => (!empty($item['MEASURE_NAME']) ? $item['MEASURE_NAME'] : '?'),
                    'price' => $item['PRICE'],
                    'quantity' => ($item['MEASURE_CODE'] !== '796' ? 1 : ceil($item['QUANTITY']))
                );
        }

        $data = CBoxberry::methodExecPost('ParselCreate', $sdata);

        if (isset($data['track'])) {
            if (Option::get(self::$moduleId, 'BB_STORE_STICKERS_LOCALLY') === 'Y') {
                $stickerUrl = CBoxberry::saveFilesFromApi($data['label'], $orderId);
            } else {
                $stickerUrl = $data['label'];
            }

            if (Option::get(self::$moduleId, 'BB_PARSELSEND') === 'Y') {
                $parselsend = CBoxberry::methodExec('ParselSend', 0, array('ImIds=' . $data['track']));
            }

            if (isset($stickerUrl, $parselsend['label'])) {
                $dbPdfLink = $stickerUrl . ' ' . $parselsend['label'];
            } elseif (isset($stickerUrl)) {
                $dbPdfLink = $stickerUrl;
            } else {
                $dbPdfLink = '';
            }

            $arFields = array(
                'ORDER_ID' => $orderId,
                'DATE_CHANGE' => date('d.m.Y H:i:s'),
                'LID' => $order['LID'],
                'TRACKING_CODE' => $data['track'],
                'PVZ_CODE' => $order['PVZ_CODE'],
                'STATUS' => '1',
                'STATUS_TEXT' => 'CREATED',
                'STATUS_DATE' => date('d.m.Y H:i:s'),
                'CHECK_REQUEST' => 'Y',
                'CHECK_REQUEST_DATE' => date('d.m.Y H:i:s'),
                'CHECK_PDF_LINK' => $dbPdfLink,
            );

            if (Option::get(self::$moduleId, 'BB_ADD_TRACK_NUMBER_INTO_SHIPMENT') === 'Y') {
                $order = Order::load($orderId);
                $shipmentCollection = $order->getShipmentCollection();
                foreach ($shipmentCollection as $shipment) {
                    if (!$shipment->isSystem()) {
                        $shipment->setFields(array(
                            'TRACKING_NUMBER' => $data['track']
                        ));
                    }
                }
                $order->save();
            }

            CBoxberryOrder::Update($orderId, $arFields);
            return $data;
        }

        if (isset($data['err'])) {
            return array('ERROR' => $data['err']);
        }

        return array('ERROR' => 'API_REQUEST_ERROR');
    }
}
?>